﻿using System;
using System.Diagnostics.Contracts;

namespace CodeContracts_InterfaceContracts
{
    [ContractClass(typeof(IPayrollCalculatorContract))]
    public interface IPayrollCalculator
    {
        double CalculatePaycheck( double baseRate, int hoursWorked );

        double DetermineBaseRate( EmployeeType employeeType );
    }

    [ContractClassFor(typeof(IPayrollCalculator))]
    public sealed class IPayrollCalculatorContract : IPayrollCalculator
    {
        double IPayrollCalculator.CalculatePaycheck( double baseRate, int hoursWorked )
        {
            Contract.Requires( baseRate > 0 );
            Contract.Requires( hoursWorked > 0 );
            Contract.Ensures( Contract.Result<double>() > 0 );

            return default( double );
        }

        double IPayrollCalculator.DetermineBaseRate( EmployeeType employeeType )
        {
            Contract.Ensures( Contract.Result<double>() > 0 );

            return default( double );
        }
    }

    public class HourlyPayrollCalculator : IPayrollCalculator {
        public double CalculatePaycheck( double baseRate, int hoursWorked )
        {            
            Contract.Requires( hoursWorked <= 50 );

            return baseRate * hoursWorked; 
        }

        public double DetermineBaseRate( EmployeeType employeeType )
        {
            var baseRate = 0d;
            switch ( employeeType )
            {
                case EmployeeType.LazyJoe:
                    baseRate = 7.5;
                    break;

                case EmployeeType.NineToFiver:
                    baseRate = 9.5;
                    break;

                case EmployeeType.GoGetter:
                    baseRate = 12.5;
                    break;

                case EmployeeType.OverAchiever:
                    baseRate = 17.5;
                    break;
            }
            return baseRate;
        }
    }

    public class SalaryPayrollCalculator : IPayrollCalculator
    {
        public double CalculatePaycheck( double baseRate, int hoursWorked )
        {
            Contract.Requires( hoursWorked <= 60 );
            return baseRate * hoursWorked;
        }

        public double DetermineBaseRate( EmployeeType employeeType )
        {
            var baseRate = 0d;
            switch ( employeeType )
            {
                case EmployeeType.LazyJoe:
                    baseRate = 17.5;
                    break;

                case EmployeeType.NineToFiver:
                    baseRate = 19.5;
                    break;

                case EmployeeType.GoGetter:
                    baseRate = 22.5;
                    break;

                case EmployeeType.OverAchiever:
                    baseRate = 27.5;
                    break;
            }
            return baseRate;
        }
    }
}